/*

Interface.java 
  -- classes implementing the GUI interface for the IAsolver applet,
     this is one example of the use of the ia_math, ia_parser, and
     ia_solver libraries,
     as part of the "ia_solver library" version 0.1beta1, 10/97

Copyright (C) 1997 Timothy J. Hickey

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the
Free Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA  02111-1307, USA.

 */


package ia_solver;

import java.awt.*;
import java.util.*; 
 
import ia_parser.Exp;
import ia_parser.ExpParser;
import ia_parser.RealIntervalTable;
/* import ia_solver.utility.*;*/

/**
 * Create a GUI interface for the Interval Arithmetic Solver.
 * the Interface class is the "application" version of the
 * Interval Arithmetic constraint solver in the ia_solver
 * package.
 * <p>
 * This application creates a window with two main text areas
 * and an assortment of control buttons and text fields. The
 * Main text area is where the user enters constraints to be
 * solved. The Result text area is where the answers are displayed.
 */
public class Interface extends Frame implements SolverGUI
{
  /* this is the initial constraint set that appears on the screen */
    static final String initialconstraints =

"                                                                                      \n" +
"   x^2 + y^2 = 25;      /* Repeatedly press the narrow button to solve! */            \n" +
"   acos(y/3)= (x-4)^2;                                                                \n" +
"                                                                                      \n" +
"                                                                                      \n" +
"                                                                                      \n" +
"/*              IAsolver version 0.1beta1,  21 October 1997                           \n" +
"  Copyright 1997 Tim Hickey, http://www.cs.brandeis.edu/~tim                          \n\n" +
"  This software is experimental and free.                                             \n" +
"  There is no warranty, use at your own risk.   */                                    \n" +
" \n"; 





 GridBagLayout gridbag = new GridBagLayout();

 MenuBar menubar;
 Menu file, help;


 TextField textFieldMessage;
 TextField textFieldMaxNarrow;
 TextField textXvar;
 TextField textYvar;

 TextArea constraints;
 TextArea varList;

 Button buttonEval;
 Button buttonStop;
 Button buttonClearVar;
 Button buttonClearCon;
 Button buttonExit;
 Button buttonPush;
 Button buttonPop;
 Button buttonPlot;
 Button buttonPlotVars;
 Button buttonAbsolve;

 Choice intervalStyle;
 Interpreter interpreter;

 Exp exp;
 RealIntervalTable vars = new RealIntervalTable();
 Vector varStack = new Vector();
 Vector constraintStack = new Vector();
 Vector plotViews = new Vector();

 Frame plot;

  /**
   * This creates the GUI for the Interval Arithmetic Constraint Solver.
   * The GUI has two major text areas. One for reading the constraints
   * and another for printing the variable bindings. It also has a textfield
   * for printing messages (status and error), and several 
   * buttons/choices/textfields for giving commands to operate 
   * on the constraints. 
   * 
   */
public Interface(boolean isApplet)
  {
    super("IAsolver 0.1beta1, 10/97");

    if (!isApplet) {
      menubar = new MenuBar();
      this.setMenuBar(menubar);
      file = new Menu("File");
      file.add(new MenuItem("Quit"));
      menubar.add(file);

      help = new Menu("Help");
      help.add(new MenuItem("About"));
      menubar.add(help);
      menubar.setHelpMenu(help);
    }
    


    // buttonpanel contains the control buttons for the IAsolver 
    Panel buttonpanel = new Panel();

        // first create several buttons
        buttonEval = new Button("Narrow");
        buttonStop = new Button("Stop");  buttonStop.disable();
        buttonClearVar = new Button("Clear Variables");
        buttonClearCon = new Button("Clear Constraints");

        buttonPush = new Button("Push");
        buttonPop = new Button("Pop");

        buttonPlot = new Button("Plot xy");
        buttonAbsolve = new Button("Absolve");

        if (isApplet)
          buttonExit = new Button("Hide");
        else
          buttonExit = new Button("Exit");

        buttonpanel.setLayout(new GridLayout(5,2,5,5));
        buttonpanel.add(buttonEval);
        buttonpanel.add(buttonStop);

	//        buttonpanel.add(buttonPush);
	//        buttonpanel.add(buttonPop);

        buttonpanel.add(buttonClearVar);
        buttonpanel.add(buttonClearCon);

	//        buttonpanel.add(buttonPlot);
        buttonpanel.add(buttonAbsolve);

        buttonpanel.add(buttonExit);


    // controlpanel contains the control panel buttons for the IAsolver 
    Panel controlpanel = new Panel();

        // first create the controls for the panel

        Label labelStyle = new Label("Interval Style:");
        intervalStyle = new Choice();
        intervalStyle.addItem("x +/- e");
        intervalStyle.addItem("[lo,hi]");

        Label labelMaxNarrow = new Label("Iterations:");
        textFieldMaxNarrow = new TextField("10",10);

        buttonPlotVars = new Button("Plot");
        textXvar = new TextField("x",10);
        Label label_vs = new Label(" versus ");
        textYvar = new TextField("y",10);

	// next we add them to the panel with appropriate constraints 
	controlpanel.setLayout(gridbag);
	Constrain.constrain(controlpanel,labelStyle,        0,0,1,1,GridBagConstraints.NONE,GridBagConstraints.EAST,0.0,0.0,0,0,0,0);
	Constrain.constrain(controlpanel,intervalStyle,     1,0,1,1,GridBagConstraints.NONE,GridBagConstraints.WEST,0.0,0.0,0,0,0,0);
	Constrain.constrain(controlpanel,labelMaxNarrow,    0,1,1,1,GridBagConstraints.NONE,GridBagConstraints.EAST,0.0,0.0,0,0,0,0);
	Constrain.constrain(controlpanel,textFieldMaxNarrow,1,1,1,1,GridBagConstraints.NONE,GridBagConstraints.WEST,0.0,0.0,0,0,0,0);
      
	Constrain.constrain(controlpanel,buttonPlotVars,    0,2,1,1,GridBagConstraints.NONE,GridBagConstraints.EAST,0.0,0.0,0,0,0,0);
	Constrain.constrain(controlpanel,textXvar,          1,2,1,1,GridBagConstraints.NONE,GridBagConstraints.EAST,0.0,0.0,0,0,0,0);
	Constrain.constrain(controlpanel,label_vs,          2,2,1,1,GridBagConstraints.NONE,GridBagConstraints.EAST,0.0,0.0,0,0,0,0);
	Constrain.constrain(controlpanel,textYvar,          3,2,1,1,GridBagConstraints.NONE,GridBagConstraints.EAST,0.0,0.0,0,0,0,0);



    // panel messagepanel contains the narrowing limit textfield and a message window
    Panel messagepanel = new Panel();
        textFieldMessage = new TextField();
        textFieldMessage.setEditable(false);
        messagepanel.setLayout(gridbag);
        Constrain.constrain(messagepanel,textFieldMessage,  0,0,1,1,GridBagConstraints.BOTH,GridBagConstraints.CENTER,1.0,1.0,0,0,0,0);


    // panel datapanel contains the varlist and the constraint area
    Panel datapanel = new Panel();
    datapanel.setLayout(gridbag);

       Label varheader = new Label("Variables");
       Label cheader = new Label("Constraints");


      // first create the list of variables
      varList = new TextArea(4,80);
      varList.setText("");
      varList.setEditable(false);

      // next create a text area for the constraints
      constraints = new TextArea(4,80);
      constraints.setText(initialconstraints);

      Constrain.constrain(datapanel,varheader,  0, 0,1,1);
      Constrain.constrain(datapanel,varList,    0, 1,1,8,GridBagConstraints.BOTH,GridBagConstraints.NORTHWEST,1.0,1.0,0,0,10,0);
      Constrain.constrain(datapanel,cheader,    0, 9,1,1);
      Constrain.constrain(datapanel,constraints,0,10,1,8,GridBagConstraints.BOTH,GridBagConstraints.NORTHWEST,1.0,1.0,0,0,0,0);
    

    this.setLayout(gridbag);
    Constrain.constrain(this,datapanel,   0,0,2,1,GridBagConstraints.BOTH,GridBagConstraints.CENTER,1.0,1.0,5,5,5,5);
    Constrain.constrain(this,messagepanel,0,1,2,1,GridBagConstraints.BOTH,GridBagConstraints.CENTER,1.0,0.0,5,5,5,5);
    Constrain.constrain(this,buttonpanel, 0,2,1,1,GridBagConstraints.NONE,GridBagConstraints.WEST,  0.0,0.0,5,5,5,5);
    Constrain.constrain(this,controlpanel,1,2,1,1,GridBagConstraints.BOTH,GridBagConstraints.EAST,  0.0,0.0,5,5,5,5);

    ThrowError.setOutput(varList);

    interpreter = new Interpreter(constraints.getText(),this);

    this.pack();

  }      
  

  public void destroy() {
    this.hide();
    if (plot != null) plot.dispose();
    if (interpreter != null) {
      interpreter.stop();
    }
    dispose();
  }

 public boolean handleEvent(Event evt)
  {
    if(evt.id == Event.WINDOW_DESTROY) 
      {this.destroy();
      }
    return super.handleEvent(evt);
  }
  
 public boolean action(Event evt, Object arg)
  {
    if(evt.target==intervalStyle) {
        displayVars();
    }
    if(arg.equals("Narrow"))
      {
        interpreter = new Interpreter(Interpreter.NARROW,this);
        interpreter.start();
      }
    
    else if(arg.equals("Stop"))
      {
	if(interpreter.isAlive()) interpreter.stop();
	buttonEval.enable();
	buttonStop.disable();
	buttonClearVar.enable();
	buttonClearCon.enable();
	buttonAbsolve.enable();
      }
    else if(arg.equals("Clear Constraints")) 
      {
        exp = null;
	constraints.setText("");      } 
    else if(arg.equals("Clear Variables")) 
      {
        varList.setText("");
        vars = new RealIntervalTable();
        if (exp != null) exp.bindVars(vars);
      } 
    //    else if(arg.equals("Push")) 
    //      {
    //        pushStack();
    //        setMessageField("PUSH: Stack contains "+sizeStack()+" elements.");
    //      }
    //    else if(arg.equals("Pop")) 
    //      {
    //        if (popStack()) {
    //          setMessageField("POP: Stack contains "+sizeStack()+" elements.");
    //	}
    //        else {
    //          setMessageField("Can't pop an empty stack!");  
    //	}
    //      }
    //    else if(arg.equals("Plot xy")) 
    //      {
    //        Frame plot = new Plot("x","y",3,getConstraints(),vars,this);
    //        plot.show();
    //      }
    else if(arg.equals("Plot")) 
      {
        setMessageField("Creating/initializing plot window");
        plot = new Plot(textXvar.getText().trim(),
                              textYvar.getText().trim(),
                              3,getConstraints(),vars,this);
	//        plot.show();
      }
    else if(arg.equals("Absolve")) 
      {
        interpreter = new Interpreter(Interpreter.ABSOLVE,this);
        interpreter.start();
      }
    else if(arg.equals("About")) {
       textFieldMessage.setText("IAsolver version 0.1beta1, copyright C Tim Hickey 1997, http://www.cs.brandeis.edu/~tim, 10/97 ");
    }
    else if(arg.equals("Hide")) this.hide();
    else if(arg.equals("Exit")) System.exit(0);
    else if(arg.equals("Quit")) System.exit(0);
    else return super.action(evt,arg);
      
    return true;
  }
  
  //  public Dimension preferredSize() {
  //     return new Dimension(400,400);
  //   }

  public int sizeStack() {
    return varStack.size();
  }

  public void pushStack() {
       varStack.addElement(vars);
       constraintStack.addElement(constraints.getText());
       vars = (RealIntervalTable) vars.clone();
       if (exp!=null) exp.bindVars(vars);
  }

  public boolean popStack() {
        if (varStack.size() > 0) {
          vars =    (RealIntervalTable) varStack.elementAt(varStack.size()-1);
          constraints.setText((String) constraintStack.elementAt(constraintStack.size()-1));
                 varStack.removeElementAt(varStack.size()-1);
          constraintStack.removeElementAt(constraintStack.size()-1);
          exp = ExpParser.parseString(constraints.getText());
          if (exp!= null) exp.bindVars(vars);
          displayVars();
          return(true);
	}
       return false;

  }

  void displayVars() {
    if (this.getIntervalStyle().equals("x +/- e"))
  	    {String var_string="";
            for(int j=0;j<vars.size(); j++)
              var_string = var_string +  "  " +
                       vars.getEnvPairString(j) + "\n";
            this.setVarList(var_string);
	    }
           else
  	    {String var_string="";
            for(int j=0;j<vars.size(); j++)
              var_string = var_string +  "  " +
                       vars.getEnvPairString2(j) + "\n";
	    //            parent.varList.setText(var_string);
            this.setVarList(var_string);
	    }
  }


public static void main(String[] args)
  {
    Frame f = new Interface(false);
    f.resize(800,600);
    f.show();
  }
  

  public void setMessageField(String s){
       textFieldMessage.setText(s);
  }

  public void setButtonStatus(String s){
    if (s.equals("NARROW")) {
	buttonStop.enable();
	buttonEval.disable();
	buttonAbsolve.disable();
	buttonClearVar.disable();
	buttonClearCon.disable();
    }
    else if (s.equals("READY")){
	buttonStop.disable();
	buttonEval.enable();
	buttonAbsolve.enable();
	buttonClearVar.enable();
	buttonClearCon.enable();
    }
    else if (s.equals("ABSOLVE")){
	buttonStop.enable();
	buttonEval.disable();
	buttonAbsolve.disable();
	buttonClearVar.disable();
	buttonClearCon.disable();

    }

  }

  public Integer getMaxNarrow(){
        try {
             return  Integer.valueOf(textFieldMaxNarrow.getText());
	 } catch (Exception e) {
             textFieldMessage.setText(
             "ERROR: narrowing value must be an integer");
              System.out.println(
                 " string was ["+
                  textFieldMaxNarrow.getText() + "]\n"+
                 "exception "+e+"\n"+
                 "max narrow must be an integer\n");
             return null;
	 }
  }


  public Exp getConstraints(){
    Exp exp;
    String old = textFieldMessage.getText();
    textFieldMessage.setText("Reading and constructing constraints... please wait");
      try {
        exp = ExpParser.parseString(constraints.getText());
        if (exp==null) {
         textFieldMessage.setText("SYNTAX ERROR in constraints....");
         return null;
        }else {
         textFieldMessage.setText(old);
         exp.bindVars(vars);
         return exp;
	}
      }
      catch (Exception parse_exception) {
         textFieldMessage.setText("SYNTAX ERROR in constraints....");
         System.out.println(parse_exception);      
         return null;
      }

  }

  public RealIntervalTable getVars() {
    return vars;
  }


  public String getIntervalStyle(){
      return intervalStyle.getSelectedItem();
  }

  public void setVarList(String s){
    varList.setText(s);
  }
    
  public void notifyNarrows(int i){
    if ((i%10)!=0) return;
       displayVars();

  }

  public void notifyFailure(){
    vars = new RealIntervalTable();
  }

  public void notifySuccess(){
  }


  };               


